@@ -1,16 +1,19 @@
-use std::fs::File;
-use std::io::Read;
-use std::io::Seek;
-use std::io::SeekFrom;
-use std::io::Write;
+use {
+    std::fs::File,
+    std::io::Read,
+    std::io::Seek,
+    std::io::SeekFrom,
+    std::io::Write,
+};
 
 
 
+// `IoSlice` supports slicing streams up to 9,000 PiB in size (`i64::max` bytes).
+//
+// the value of `begin`, `length`, `remaining`, `begin + length` will never be greater than `std::max::i64`. these
+// invariants are guarenteed by the io-slice constructor.
 #[derive(Debug)]
 pub struct IoSlice<T> where T: Seek {
-    // note: the value of `begin`, `length`, `remaining`, and `begin + length` will never be greater than the value of `std::i64::max`
-    // note: `length` and `remaining` will never be greater than `std::usize::max`
-
     underlying: T,
     begin:      u64,
     length:     u64,
@@ -20,38 +23,25 @@ pub struct IoSlice<T> where T: Seek {
 impl<T> IoSlice<T> where T: Seek {
     pub fn new(mut source: T, begin: u64, length: u64) -> Result<IoSlice<T>, std::io::Error> {
         // :: check invariants
-        let u64_max   = std::i64::MAX as u64;
-        let usize_max = std::usize::MAX as u64;
+        let i64_max = std::i64::MAX as u64;
 
-        if begin > u64_max || length > usize_max || begin + length > u64_max {
+        if begin > i64_max || length > i64_max || begin + length > i64_max {
             return Err(std::io::ErrorKind::InvalidInput.into());
         }
 
 
         // :: attempt to seek to the requested position.
+        //        if our request to seek to `begin` does not place us at `begin`, this stream is "invalid".
         let seek = SeekFrom::Start(begin);
 
-        match source.seek(seek) {
-            Ok(position) => {
-                // if our request to seek to `begin` does not place us at `begin`, this stream is "invalid".
-                match position == begin {
-                    true  => Ok(()),
-                    false => Err(std::io::ErrorKind::InvalidInput.into()),
-                }
-            },
-            Err(error) => {
-                Err(error)
-            },
-        }?;
+        if source.seek(seek)? == begin {
+            let underlying = source;
+            let remaining  = length;
 
-
-        // :: ok
-        Ok(IoSlice {
-            underlying: source,
-            begin:      begin,
-            length:     length,
-            remaining:  length,
-        })
+            Ok(IoSlice { underlying, begin, length, remaining })
+        } else {
+            Err(std::io::ErrorKind::InvalidInput.into())
+        }
     }
 
     pub fn len(&self) -> u64 {
@@ -88,7 +78,9 @@ impl<T> Seek for IoSlice<T> where T: Seek {
         // if the new requested position is in bounds. seek to it, and make sure that the new position is the one we
         // requested.
         if absolute >= self.begin && absolute <= self.begin + self.length {
-            if self.underlying.seek(SeekFrom::Start(absolute))? == absolute {
+            let seek = SeekFrom::Start(absolute);
+
+            if self.underlying.seek(seek)? == absolute {
                 let new = absolute - self.begin;
 
                 self.remaining = self.length - new;
@@ -109,24 +101,29 @@ impl<T> Read for IoSlice<T> where T: Read + Seek {
         let remaining   = std::cmp::min(self.remaining, std::usize::MAX as u64) as usize;
         let request     = std::cmp::min(remaining, buffer.len());
         let actual      = self.underlying.read(&mut buffer[..request])?;
+
         self.remaining -= actual as u64;
 
         Ok(actual)
     }
 
     fn read_to_end(&mut self, buffer: &mut Vec<u8>) -> Result<usize, std::io::Error> {
+        if self.remaining > std::usize::MAX as u64 {
+            return Err(std::io::ErrorKind::InvalidInput.into())
+        }
+
         let length    = buffer.len();
         let remaining = self.remaining as usize;
 
         buffer.reserve(remaining);
 
         unsafe {
-            let ptr   = buffer.as_mut_ptr().offset(length as isize);
-            let slice = std::slice::from_raw_parts_mut(ptr, remaining);
+            let pointer = buffer.as_mut_ptr().add(length);
+            let slice   = std::slice::from_raw_parts_mut(pointer, remaining);
 
             self.underlying.read_exact(slice)?;
-
             buffer.set_len(length + remaining);
+
             self.remaining = 0;
         }
 
@@ -141,11 +138,20 @@ impl<T> Write for IoSlice<T> where T: Write + Seek {
         }
 
         let actual = self.underlying.write(buffer)?;
+
         self.remaining -= actual as u64;
 
         Ok(actual)
     }
 
+    fn write_all(&mut self, buffer: &[u8]) -> Result<(), std::io::Error> {
+        if buffer.len() as u64 > self.remaining {
+            return Err(std::io::ErrorKind::UnexpectedEof.into());
+        }
+
+        self.underlying.write_all(buffer)
+    }
+
     fn flush(&mut self) -> Result<(), std::io::Error> {
         self.underlying.flush()
     }
